# -*- coding: utf-8 -*-

"""
A cross-platform CLI tool for lightning-fast AWS Console URL opener.
"""


from ._version import __version__

__short_description__ = "A cross-platform CLI tool for lightning-fast AWS Console URL opener."
__license__ = "GPL"
__author__ = "Sanhe Hu"
__author_email__ = "husanhe@gmail.com"
__github_username__ = "MacHu-GWU"
